import React from 'react';
import { ReactWidget } from '@jupyterlab/apputils';
import { ILabShell } from '@jupyterlab/application';
import { INotebookTracker } from '@jupyterlab/notebook';
import { IEditorTracker } from '@jupyterlab/fileeditor';
import { IDocumentManager } from '@jupyterlab/docmanager';
import { TranslationBundle } from '@jupyterlab/translation';
/**
 * Parent element for extension. Listens to user actions outside of
 * extension interface and emits into so interface can act accordingly
 *
 * @export
 * @class CommentWidget
 * @typedef {CommentWidget}
 * @extends {ReactWidget}
 */
export declare class CommentWidget extends ReactWidget {
    /**
     * Object that tracks if currently openned file was changed, we
     * need this to update current file path and name
     *
     * @private
     * @type {ILabShell}
     */
    private labShell;
    /**
     * Object that stores information about oppened files, like their
     * paths and names
     *
     * @private
     * @type {IDocumentManager}
     */
    private docmanager;
    /**
     * Object that tracks events in notebooks, wee need this to
     * check which cell is currently focused, to link comments
     * to it
     *
     * @private
     * @type {INotebookTracker}
     */
    private notebookTracker;
    /**
     * Object that tracks events in documents, we need this to
     * check which where user's cursor is placed to link comments
     * to those lines
     *
     * @private
     * @type {IDocumentManager}
     */
    private editorTracker;
    /**
     * Object trat translated interface text into languages for
     * which translation was provided
     *
     * @private
     * @type {TranslationBundle}
     */
    private trans;
    /**
     * Parameters about current user, openned file and comments linked to it
     *
     * @type {{
     *   comments: [] list of comments linked to a file
     *   rootPath: string full path of directory where user started jupyter
     *   filePath: string path to the currently openned file
     *   fileName: string name of the currently openned file
     *   username: string username of user that interacts with jupyter
     *   fullUsername: string real name of user
     *   commentTimeouts: { [key: string]: NodeJS.Timeout; } timeout that prevents highligh
     *   effect from triggering multiple times
     * }}
     */
    state: {
        comments: any[];
        rootPath: string;
        filePath: string;
        fileName: string;
        username: string;
        fullUsername: string;
        commentTimeouts: {
            [key: string]: NodeJS.Timeout;
        };
    };
    /**
     * Creates an instance of CommentWidget.
     *
     * @constructor
     * @param {ILabShell} labShell
     * @param {IDocumentManager} docmanager
     * @param {INotebookTracker} notebookTracker
     * @param {IEditorTracker} editorTracker
     * @param {TranslationBundle} trans
     */
    constructor(labShell: ILabShell, docmanager: IDocumentManager, notebookTracker: INotebookTracker, editorTracker: IEditorTracker, trans: TranslationBundle);
    /**
     * waits until one of the widgets in the jupyter interface
     * becomes active, loads comments for that file and adds
     * an event handler that listens for active file changes
    */
    initializeWidget(): void;
    /**
     * if comment-id is in url, open comment widget
     * and highlight a comment with requested id
     */
    processCommentLink(): void;
    /**
     * checks if any widget is active, if it is, extracts its path
     * and sets according state variables
     */
    trySettingLocalPath(): void;
    /**
     * triggers when user selects another widget (file) in
     * the interface, loads comments for that widget
     */
    subscribeToShellChanges(): void;
    /**
     * opens comments widget if is is collapsed
     * searches for a comment with specified id
     * if search is successfull, focuses that comment
     * and adds a glow animation to highlight it
     *
     * @param {string} commentId
     */
    focusComment(commentId: string): void;
    /**
     * returns a container with comment icon, that should be inserted
     * into editor / notebook as an indicator that there is a comment there
     *
     * @param {string} commentId
     * @param {string} containerElement tag of an element that should contain indicator icon
     * @param {string} className class name of an icon container
     * @returns {*}
     */
    getIconContainer(commentId: string, containerElement: string, className: string): HTMLElement;
    /** Rerenders cell comment icons on cell execution */
    subscribeToCellExecution(): void;
    /**
     * If comments are linked to current notebook widget
     * add icons that indicate which cells were commented on
     */
    addCommentIconsToCells(): void;
    /**
     * If comments are linked to current editor widget
     * add icons that indicate which lines were commented on
     */
    addCommentIconsToEditorLines(): void;
    /**
     * extracts and sets username for current user, it defines new comment author as well
     * as which actions current user can perform on existing comments
     */
    setUsername(): void;
    /** Sets where user started jupyter, to resolve full path of files */
    setRootPath(): void;
    /** loads comments for current widget, updates them in the interface */
    updateComments(): void;
    render(): React.JSX.Element;
}
