import React from 'react';
import { ReactWidget } from '@jupyterlab/apputils';
import { LabIcon } from '@jupyterlab/ui-components';
import { PageConfig } from '@jupyterlab/coreutils';
import { requestAPI } from './handler';
import { Panel } from './panel';
import commentSvg from '../style/icons/comment-icon.svg';
/**
 * Icon that is used on the right side menu and to the left of
 * cells / editor lines which contain comments
 *
 * @type {*}
 */
const commentIcon = new LabIcon({
    name: 'CommentIcon',
    svgstr: commentSvg
});
/**
 * Parent element for extension. Listens to user actions outside of
 * extension interface and emits into so interface can act accordingly
 *
 * @export
 * @class CommentWidget
 * @typedef {CommentWidget}
 * @extends {ReactWidget}
 */
export class CommentWidget extends ReactWidget {
    /**
     * Creates an instance of CommentWidget.
     *
     * @constructor
     * @param {ILabShell} labShell
     * @param {IDocumentManager} docmanager
     * @param {INotebookTracker} notebookTracker
     * @param {IEditorTracker} editorTracker
     * @param {TranslationBundle} trans
     */
    constructor(labShell, docmanager, notebookTracker, editorTracker, trans) {
        super();
        /**
         * Parameters about current user, openned file and comments linked to it
         *
         * @type {{
         *   comments: [] list of comments linked to a file
         *   rootPath: string full path of directory where user started jupyter
         *   filePath: string path to the currently openned file
         *   fileName: string name of the currently openned file
         *   username: string username of user that interacts with jupyter
         *   fullUsername: string real name of user
         *   commentTimeouts: { [key: string]: NodeJS.Timeout; } timeout that prevents highligh
         *   effect from triggering multiple times
         * }}
         */
        this.state = {
            comments: [],
            rootPath: '',
            filePath: '',
            fileName: '',
            username: 'guest',
            fullUsername: '',
            commentTimeouts: {}
        };
        this.id = 'comment-widget';
        this.title.icon = commentIcon;
        this.labShell = labShell;
        this.docmanager = docmanager;
        this.notebookTracker = notebookTracker;
        this.editorTracker = editorTracker;
        this.trans = trans;
        this.updateComments = this.updateComments.bind(this);
        this.subscribeToCellExecution();
        this.initializeWidget();
    }
    /**
     * waits until one of the widgets in the jupyter interface
     * becomes active, loads comments for that file and adds
     * an event handler that listens for active file changes
    */
    initializeWidget() {
        this.setUsername();
        this.setRootPath();
        const interval = setInterval(() => {
            if (this.state.filePath) {
                clearInterval(interval);
                this.subscribeToShellChanges();
                this.updateComments();
                setTimeout(() => {
                    this.processCommentLink();
                }, 500);
            }
            else {
                this.trySettingLocalPath();
            }
        }, 250);
    }
    /**
     * if comment-id is in url, open comment widget
     * and highlight a comment with requested id
     */
    processCommentLink() {
        const urlParams = new URLSearchParams(window.location.search);
        const commentId = urlParams.get('comment-id');
        const filePath = window.location.pathname.split('lab/tree/')[1];
        if (commentId && filePath) {
            this.docmanager.openOrReveal(filePath);
            this.focusComment(commentId);
        }
    }
    /**
     * checks if any widget is active, if it is, extracts its path
     * and sets according state variables
     */
    trySettingLocalPath() {
        var _a;
        const currentWidget = this.labShell.currentWidget;
        if (currentWidget) {
            const localPath = (_a = this.docmanager.contextForWidget(currentWidget)) === null || _a === void 0 ? void 0 : _a.localPath;
            if (localPath) {
                this.state.filePath = this.state.rootPath + '/' + localPath;
                this.state.fileName = localPath.split('/').slice(-1)[0];
            }
        }
    }
    /**
     * triggers when user selects another widget (file) in
     * the interface, loads comments for that widget
     */
    subscribeToShellChanges() {
        this.labShell.currentChanged.connect((sender, args) => {
            var _a;
            const currentWidget = args.newValue;
            if (!currentWidget) {
                return;
            }
            const localPath = (_a = this.docmanager.contextForWidget(currentWidget)) === null || _a === void 0 ? void 0 : _a.localPath;
            if (localPath) {
                this.state.filePath = this.state.rootPath + '/' + localPath;
                this.state.fileName = localPath.split('/').slice(-1)[0];
                this.updateComments();
            }
            else {
                this.state.filePath = '';
                this.updateComments();
            }
        });
    }
    /**
     * opens comments widget if is is collapsed
     * searches for a comment with specified id
     * if search is successfull, focuses that comment
     * and adds a glow animation to highlight it
     *
     * @param {string} commentId
     */
    focusComment(commentId) {
        const container = document.querySelector('.jp-comments-comments-container');
        const comment = container === null || container === void 0 ? void 0 : container.querySelector('[data-comment-id="' + commentId + '"]');
        if (comment) {
            comment.scrollIntoView({ behavior: 'smooth', block: 'start' });
            comment.classList.add('jp-comments-glowing-border');
            this.labShell.activateById('comment-widget');
            this.labShell.expandRight();
            if (commentId in this.state.commentTimeouts) {
                clearTimeout(this.state.commentTimeouts[commentId]);
            }
            const timeoutId = setTimeout(() => {
                comment.classList.remove('jp-comments-glowing-border');
            }, 3000);
            this.state.commentTimeouts[commentId] = timeoutId;
        }
    }
    /**
     * returns a container with comment icon, that should be inserted
     * into editor / notebook as an indicator that there is a comment there
     *
     * @param {string} commentId
     * @param {string} containerElement tag of an element that should contain indicator icon
     * @param {string} className class name of an icon container
     * @returns {*}
     */
    getIconContainer(commentId, containerElement, className) {
        const iconContainer = document.createElement(containerElement);
        iconContainer.className = className;
        iconContainer.dataset.commentId = commentId;
        iconContainer.onclick = (ev) => {
            const target = ev.target;
            const targetDiv = target.closest('[data-comment-id]');
            const commentId = targetDiv === null || targetDiv === void 0 ? void 0 : targetDiv.dataset.commentId;
            if (commentId) {
                this.focusComment(commentId);
            }
        };
        commentIcon.element({
            container: iconContainer
        });
        return iconContainer;
    }
    /** Rerenders cell comment icons on cell execution */
    subscribeToCellExecution() {
        const onExecutionCountChanged = (_, changed) => {
            if (changed.name === 'executionCount' && changed.newValue) {
                this.addCommentIconsToCells();
            }
        };
        this.notebookTracker.activeCellChanged.connect((_, newActiveCell) => {
            newActiveCell === null || newActiveCell === void 0 ? void 0 : newActiveCell.model.stateChanged.connect(onExecutionCountChanged);
        });
    }
    /**
     * If comments are linked to current notebook widget
     * add icons that indicate which cells were commented on
     */
    addCommentIconsToCells() {
        var _a;
        const className = 'jp-comments-cell-icon';
        const panel = (_a = this.notebookTracker.currentWidget) === null || _a === void 0 ? void 0 : _a.content;
        if (!panel) {
            return;
        }
        const commentIds = {};
        this.state.comments.map(c => {
            const cellId = c.cell_id;
            const commentId = c.id;
            if (cellId && !c.parent_id && !c.resolved) {
                commentIds[cellId] = commentId;
            }
        });
        panel === null || panel === void 0 ? void 0 : panel.widgets.map(w => {
            var _a;
            (_a = w.node.querySelector('.' + className)) === null || _a === void 0 ? void 0 : _a.remove();
            const cellId = w.model.id;
            if (cellId in commentIds) {
                // if icon already exists remove it
                const commentId = commentIds[cellId];
                const cell = w.node.querySelector('.jp-InputArea-prompt');
                const iconContainer = this.getIconContainer(commentId, 'div', className);
                cell === null || cell === void 0 ? void 0 : cell.appendChild(iconContainer);
            }
        });
    }
    /**
     * If comments are linked to current editor widget
     * add icons that indicate which lines were commented on
     */
    addCommentIconsToEditorLines() {
        var _a;
        const className = 'jp-comments-editor-line-icon';
        const editorContainer = (_a = this.editorTracker.currentWidget) === null || _a === void 0 ? void 0 : _a.content;
        const editor = editorContainer === null || editorContainer === void 0 ? void 0 : editorContainer.editor;
        if (!editor) {
            return;
        }
        const elements = editorContainer === null || editorContainer === void 0 ? void 0 : editorContainer.node.querySelectorAll('.' + className);
        // removes existing comment icons
        if (elements) {
            for (let i = 0; i < (elements === null || elements === void 0 ? void 0 : elements.length); i++) {
                elements[i].remove();
            }
        }
        // places new icons
        const commentIds = {};
        this.state.comments.map(c => {
            const editorLine = c.editor_line;
            const editorLineText = c.editor_line_text;
            const commentId = c.id;
            if (!c.parent_id && !c.resolved) {
                commentIds[editorLine] = [commentId, editorLineText];
            }
        });
        Object.keys(commentIds).map(line => {
            const editorLine = Number(line);
            const commentId = commentIds[editorLine][0];
            const editorLineText = commentIds[editorLine][1];
            try {
                const offset = editor.getOffsetAt({ line: editorLine, column: 0 });
                const editorText = editor.getTokenAt(offset).value;
                if (editorLineText === editorText) {
                    const editorLineEl = editorContainer.node.querySelectorAll('.cm-lineNumbers .cm-gutterElement')[editorLine + 1];
                    const iconContainer = this.getIconContainer(commentId, 'span', className);
                    editorLineEl === null || editorLineEl === void 0 ? void 0 : editorLineEl.prepend(iconContainer);
                }
                else {
                    //text at commentLine has changed, reset comment to line 0
                }
            }
            catch (ex) {
                if (ex instanceof RangeError) {
                    // line doesn't exist anymore, delete comment?
                }
                else {
                    throw ex;
                }
            }
        });
    }
    /**
     * extracts and sets username for current user, it defines new comment author as well
     * as which actions current user can perform on existing comments
     */
    setUsername() {
        requestAPI('username', { method: 'GET' })
            .then(reply => {
            if (reply.error) {
                console.error(`Error on GET /username: ${reply.error}`);
            }
            else {
                this.state.username = reply.username;
                this.state.fullUsername = reply.full_username;
            }
        })
            .catch(reason => {
            console.log(reason.response);
            console.error(`Error on GET /jupyterhub-comments-server/username.\n${reason.message.error}`);
        });
    }
    /** Sets where user started jupyter, to resolve full path of files */
    setRootPath() {
        requestAPI('resolve-path', {
            method: 'POST',
            body: JSON.stringify({ 'root_path': PageConfig.getOption('serverRoot') })
        })
            .then(reply => {
            if (reply.error) {
                console.error(`Error on POST /resolve-path: ${reply.error}`);
            }
            else {
                this.state.rootPath = reply.resolved_path;
            }
        })
            .catch(reason => {
            console.log(reason.response);
            console.error(`Error on POST /jupyterhub-comments-server/resolve-path.\n${reason.message.error}`);
        });
    }
    /** loads comments for current widget, updates them in the interface */
    updateComments() {
        if (!this.state.filePath) {
            this.trySettingLocalPath();
        }
        const dataToSend = {
            file_path: this.state.filePath
        };
        requestAPI('comments', {
            body: JSON.stringify(dataToSend),
            method: 'POST'
        })
            .then(reply => {
            if (reply.error) {
                console.error(`Error on POST /comments: ${reply.error}`);
                return [];
            }
            else {
                this.state.comments = reply.comments || [];
                this.update();
                setTimeout(() => {
                    this.addCommentIconsToCells();
                    this.addCommentIconsToEditorLines();
                }, 50);
            }
        })
            .catch(reason => {
            console.log(reason.response);
            console.error(`Error on POST /jupyterhub-comments-server/comments ${dataToSend}.\n${reason.message.error}`);
        });
    }
    render() {
        return (React.createElement(Panel, { notebookTracker: this.notebookTracker, editorTracker: this.editorTracker, trans: this.trans, username: this.state.username, fullUsername: this.state.fullUsername, filePath: this.state.filePath, fileName: this.state.fileName, comments: this.state.comments, updateComments: this.updateComments }));
    }
}
