import React from 'react';
import { INotebookTracker } from '@jupyterlab/notebook';
import { TranslationBundle } from '@jupyterlab/translation';
import { IEditorTracker } from '@jupyterlab/fileeditor';
import '../style/comment.css';
/**
 * Type defining comment fields
 *
 * @export
 * @interface ICommentType
 * @typedef {ICommentType}
 */
export interface ICommentType {
    id: string;
    file_path: string;
    cell_id: string;
    editor_line: number;
    editor_line_text: string;
    username: string;
    full_username: string;
    text: string;
    timestamp: number;
    updated_timestamp: number;
    resolved: number;
    parent_id: string;
    file_owner: string;
}
/**
 * Element that defines a comment thread, which has parentComment
 * and any number of children comments
 *
 * @param {{
 *   notebookTracker: INotebookTracker;
 *   editorTracker: IEditorTracker;
 *   trans: TranslationBundle;
 *   parentComment: ICommentType;
 *   childrenComments: ICommentType[];
 *   fullUsername: string;
 *   username: string;
 *   hideResolved: boolean;
 *   updateComments: () => void;
 * }} param0
 * @param {INotebookTracker} param0.notebookTracker, object that tracks changes in notebooks
 * @param {IEditorTracker} param0.editorTracker, object that tracks changes in editors
 * @param {TranslationBundle} param0.trans, translation object
 * @param {ICommentType} param0.parentComment, comment that thread is based on
 * @param {{}} param0.childrenComments, comment that are replies to parentComment
 * @param {string} param0.fullUsername, real name of a user
 * @param {string} param0.username, system username of user interacting with jupyter
 * @param {boolean} param0.hideResolved, if resolved parentComments should be hidden
 * @param {() => void} param0.updateComments, function to pull comments and rerender inteface
 * @returns {React.JSX.Element}
 */
export declare const CommentThread: ({ notebookTracker, editorTracker, trans, parentComment, childrenComments, fullUsername, username, hideResolved, updateComments }: {
    notebookTracker: INotebookTracker;
    editorTracker: IEditorTracker;
    trans: TranslationBundle;
    parentComment: ICommentType;
    childrenComments: ICommentType[];
    fullUsername: string;
    username: string;
    hideResolved: boolean;
    updateComments: () => void;
}) => React.JSX.Element;
/**
 * Element that represents a single comment either parent or child one
 *
 * @param {{
 *   trans: TranslationBundle;
 *   comment: ICommentType;
 *   currentUsername: string;
 *   updateComments: () => void;
 *   handleReplyClick: (event: React.MouseEvent<HTMLDivElement>) => void;
 * }} param0
 * @param {TranslationBundle} param0.trans, translation object
 * @param {ICommentType} param0.comment, comment object
 * @param {string} param0.currentUsername, username of a user that started jupyter
 * @param {() => void} param0.updateComments, function to pull comments and rerender inteface
 * @param {(event: React.MouseEvent<HTMLDivElement>) => void} param0.handleReplyClick, function
 * that shows add comment form when reply button is clicked on parent comment
 * @returns {JSX.Element}
 */
export declare const Comment: ({ trans, comment, currentUsername, updateComments, handleReplyClick }: {
    trans: TranslationBundle;
    comment: ICommentType;
    currentUsername: string;
    updateComments: () => void;
    handleReplyClick: (event: React.MouseEvent<HTMLDivElement>) => void;
}) => JSX.Element;
